package com.utic.center.utic.ptis.server.dao.mapper;

import com.utic.center.utic.ptis.server.dto.IndigoStatsDataDto;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

@Mapper
public interface UticPtisServerMapper {

    String findLastRegDateCurrLinkSt();
    String findLastRegDateEventId();
    String findLastRegDateOccurId();

    List<IndigoStatsDataDto> findAllIndigoStats();
    int insertIndigoStats(@Param("ifId") String ifId, @Param("agency") String agency, @Param("info") String info, @Param("period") String period, @Param("countSnd") String countSnd);
    int mergeIndigoStats(@Param("ifId") String ifId, @Param("agency") String agency, @Param("info") String info, @Param("period") String period, @Param("countSnd") String countSnd);

    String findOneSysNow();
    int insertCurlinkstPoliceLog();
    int deleteRcvLinkTrafficTmp();  // none return value
    int insertRcvLinkTrafficTmp();
    int deleteRcvLinkTraffic();

    String findOneMaxEventRegDate();
    int insertEventidPoliceLog(@Param("fromDate") String fromDate, @Param("toDate") String toDate);
    int insertEventidPoliceLogTmp(@Param("fromDate") String fromDate, @Param("toDate") String toDate);

    String findOneMaxIncdRegDate();
    int insertOccuridPoliceLog(@Param("fromDate") String fromDate, @Param("toDate") String toDate);
    int insertOccuridPoliceLogTmp(@Param("fromDate") String fromDate, @Param("toDate") String toDate);

    int deleteEventidPoliceLog();
    int deleteEventidPoliceLogTmp();
    int deleteOccuridPoliceLog();
    int deleteOccuridPoliceLogTmp();
}
